@@ -48,14 +48,15 @@ codeGeno <- function(gpData,impute=FALSE,impute.type=c("random","family","beagle
   orgFormat <- class(gpData)
   # check for class 'gpData'
   if(class(gpData)=="gpData"){
-    if(is.null(gpData$geno)) stop("no genotypic data available") else res <- gpData$geno
+    if(is.null(gpData$geno)) stop("no genotypic data available")
     # family information (population structure) for genotypic data
     # drop unused levels
     if(is.factor(gpData$covar$family)) {
-      popStruc <- droplevels(gpData$covar$family[gpData$covar$genotyped])
+      popStruc <- as.character(droplevels(gpData$covar$family[gpData$covar$genotyped]))
     } else {
-      popStruc <- gpData$covar$family[gpData$covar$genotyped]
+      popStruc <- as.character(gpData$covar$family[gpData$covar$genotyped])
     }
+    names(popStruc) <- gpData$covar$id[gpData$covar$genotyped]
     if(gpData$info$codeGeno & !is.null(label.heter))
       if(is.function(label.heter)){
         warning("assuming heterozygous genotypes coded as 1. Use 'label.heter' to specify if that is not the case")
@@ -69,22 +70,24 @@ codeGeno <- function(gpData,impute=FALSE,impute.type=c("random","family","beagle
     res <- gpData
     popStruc <- NULL
     gpData <- list(geno=res, info=list(map.unit="NA", codeGeno=FALSE))
+    rm(res)
     gpData$map <- NULL
   }
   #  catch errors
   if(check){
-    if(class(res)!= "data.frame" & class(res) != "matrix") stop("wrong data format")
-    if(any(colMeans(is.na(res))==1)) warning("markers with only missing values in data")
-    if(length(reference.allele)>1 & length(reference.allele)!=ncol(res)) stop("'reference allele' should be of length 1 or match the number of markers")
-    if(class(reference.allele) != mode(res)) stop("'reference allele' should be of class character")
+    if(class(gpData$geno)!= "data.frame" & class(gpData$geno) != "matrix") stop("wrong data format")
+    if(any(colMeans(is.na(gpData$geno))==1)) warning("markers with only missing values in data")
+    if(length(reference.allele)>1 & length(reference.allele)!=ncol(gpData$geno)) stop("'reference allele' should be of length 1 or match the number of markers")
+    if(class(reference.allele) != mode(gpData$geno)) stop("'reference allele' should be of class character")
   }
   # number of genotypes
-  n <- nrow(res)
+  n <- nrow(gpData$geno)
 
   # keep names of data object
-  cnames <- colnames(res)
-  rnames <- rownames(res)
-  res <- matrix(unlist(res),nrow=n)
+  cnames <- colnames(gpData$geno)
+  rnames <- rownames(gpData$geno)
+  popStruc <- popStruc[rnames]
+  gpData$geno <- matrix(unlist(gpData$geno),nrow=n)
   # tester control
   if(!is.null(tester)){
     if(length(tester)>1) stop("Only one tester is allowed for this function\n")
@@ -105,7 +108,7 @@ codeGeno <- function(gpData,impute=FALSE,impute.type=c("random","family","beagle
   }
 
   # use same reference allele for all markers if not specified differently
-  if(reference.allele[1]!="minor" & length(reference.allele)==1)     reference.allele <- rep(reference.allele,ncol(res))
+  if(reference.allele[1]!="minor" & length(reference.allele)==1)     reference.allele <- rep(reference.allele,ncol(gpData$geno))
   knames <- cnames %in% keep.list
 
   #============================================================
@@ -114,23 +117,21 @@ codeGeno <- function(gpData,impute=FALSE,impute.type=c("random","family","beagle
 
   if(!is.null(nmiss)){
     if(nmiss<0 | nmiss>1) stop("'nmiss' have to be in [0,1]")
-    which.miss <- apply(is.na(res),2,mean,na.rm=TRUE)<=nmiss | knames
-    res <- res[,which.miss]
+    which.miss <- apply(is.na(gpData$geno),2,mean,na.rm=TRUE)<=nmiss | knames
+    gpData$geno <- gpData$geno[,which.miss]
     if(!(reference.allele[1]=="minor" | reference.allele[1]=="keep"))  reference.allele <- reference.allele[which.miss]
     if (verbose) cat("   step 1  :", sum(!which.miss),"marker(s) removed with >",nmiss*100,"% missing values \n")
     cnames <- cnames[which.miss]; knames <- knames[which.miss]
     # update map
-    gpData$geno <- gpData$geno[, which.miss]
     if(!is.null(gpData$map)) gpData$map <- gpData$map[which.miss,]
     rm(which.miss)
-  } else if (any(colMeans(is.na(res))==1)){
-    which.miss <- colMeans(is.na(res))!=1
-    res <- res[,which.miss]
+  } else if (any(colMeans(is.na(gpData$geno))==1)){
+    which.miss <- colMeans(is.na(gpData$geno))!=1
+    gpData$geno <- gpData$geno[,which.miss]
     if(!(reference.allele[1]=="minor" | reference.allele[1]=="keep"))  reference.allele <- reference.allele[which.miss]
     if (verbose) cat("   step 1  :", sum(!which.miss),"marker(s) removed with only missing values \n")
     cnames <- cnames[which.miss]; knames <- knames[which.miss]
     # update map
-    gpData$geno <- gpData$geno[, which.miss]
     if(!is.null(gpData$map)) gpData$map <- gpData$map[which.miss,]
     rm(which.miss)
   } else {
@@ -144,16 +145,16 @@ codeGeno <- function(gpData,impute=FALSE,impute.type=c("random","family","beagle
   if (verbose) cat("   step 2  : Recoding alleles \n")
   if(gpData$info$codeGeno) {
     if(reference.allele[1]=="minor"){
-      afCols <- (1:ncol(res))[colMeans(res, na.rm=TRUE)>1]
-      res[, afCols] <-  rep(1, nrow(res)) %*% t(rep(2, length(afCols))) - res[, afCols]
+      afCols <- (1:ncol(gpData$geno))[colMeans(gpData$geno, na.rm=TRUE)>1]
+      gpData$geno[, afCols] <-  rep(1, nrow(gpData$geno)) %*% t(rep(2, length(afCols))) - gpData$geno[, afCols]
       # inititialize report list
       if(print.report){
-        alleles <- apply(res,2,table,useNA="no")
+        alleles <- apply(gpData$geno,2,table,useNA="no")
         major.allele <- function(x) names(which.max(x[!names(x) %in% label.heter]))
         minor.allele <- function(x) names(which.min(x[!names(x) %in% label.heter]))
         if(class(alleles) != "list"){
           mytable <- function(x,...) as.data.frame(table(x,...),stringsAsFactors=FALSE)
-          alleles <- apply(res,2,mytable,useNA="no")
+          alleles <- apply(gpData$geno,2,mytable,useNA="no")
           major.allele <- function(x) x$x[which.max(x$Freq)]
           minor.allele <- function(x) x$x[which.min(x$Freq)]
         }
@@ -175,7 +176,7 @@ codeGeno <- function(gpData,impute=FALSE,impute.type=c("random","family","beagle
         } else {
           if (is.function(label.heter)){                          # multiple labels for heterozygous values
             is.heter <- label.heter
-            label.heter <- unique(res[which(is.heter(res),arr.ind=TRUE)])
+            label.heter <- unique(gpData$geno[which(is.heter(gpData$geno),arr.ind=TRUE)])
           } else stop("label.heter must be a character string or a function")
         }
         # make sure that NA is not in label.heter
@@ -185,12 +186,12 @@ codeGeno <- function(gpData,impute=FALSE,impute.type=c("random","family","beagle
 
       # inititialize report list
       if(print.report){
-        alleles <- apply(res,2,table,useNA="no")
+        alleles <- apply(gpData$geno,2,table,useNA="no")
         major.allele <- function(x) names(which.max(x[!names(x) %in% label.heter]))
         minor.allele <- function(x) names(which.min(x[!names(x) %in% label.heter]))
         if(class(alleles) != "list"){
           mytable <- function(x,...) as.data.frame(table(x,...),stringsAsFactors=FALSE)
-          alleles <- apply(res,2,mytable,useNA="no")
+          alleles <- apply(gpData$geno,2,mytable,useNA="no")
           major.allele <- function(x) x$x[which.max(x$Freq)]
           minor.allele <- function(x) x$x[which.min(x$Freq)]
         }
@@ -210,16 +211,16 @@ codeGeno <- function(gpData,impute=FALSE,impute.type=c("random","family","beagle
       }
 
       # apply function on whole genotypic data
-      res <- apply(as.matrix(res),2,codeNumeric)
+      gpData$geno <- apply(as.matrix(gpData$geno),2,codeNumeric)
 
       # set heterozygous genotypes as 1
-      res[res %in% label.heter] <- 1
-      res <- matrix(as.numeric(res),nrow=n)
+      gpData$geno[gpData$geno %in% label.heter] <- 1
+      gpData$geno <- matrix(as.numeric(gpData$geno),nrow=n)
     } else { #if reference.alle !="minor"
       count.ref.alleles <- function(x,ref){
    	    sum(x==ref)		
       }
-      if(mode(res) == "character"){
+      if(mode(gpData$geno) == "character"){
         recode.by.ref.alleles <- function(x){
 	      ref <- substr(x[1],1,1)
 	      x <- x[-1]
@@ -241,20 +242,20 @@ codeGeno <- function(gpData,impute=FALSE,impute.type=c("random","family","beagle
         unique(xx)[unique(xx) != x[1] & !is.na(unique(xx))]
       }
       if(any(is.na(reference.allele))){
-        ref.tab <- lapply(data.frame(res[,is.na(reference.allele)]), table)
+        ref.tab <- lapply(data.frame(gpData$geno[,is.na(reference.allele)]), table)
         ref.all <- unlist(lapply(ref.tab, names))[seq(1, by=2, length.out=length(ref.tab))]
         ref.all <- ifelse(unlist(ref.tab)[seq(1, by=2, length.out=length(ref.tab))] > unlist(ref.tab)[seq(2, by=2, length.out=length(ref.tab))],
                           ref.all, unlist(lapply(ref.tab, names))[seq(2, by=2, length.out=length(ref.tab))])
         reference.allele[is.na(reference.allele)] <- ref.all
         rm(ref.all, ref.tab)
       }
-      res_ref <- rbind(reference.allele,res)
-      res <- apply(res_ref,2,recode.by.ref.alleles)
-      rm(res_ref)
+      res_ref <- rbind(reference.allele,gpData$geno)
+      gpData$geno <- apply(res_ref,2,recode.by.ref.alleles)
       if(print.report){
         major <- reference.allele
         minor <-  apply(res_ref,2,get.nonref.allele)
       }
+      rm(res_ref)
     }
   }
 
@@ -263,8 +264,8 @@ codeGeno <- function(gpData,impute=FALSE,impute.type=c("random","family","beagle
   #============================================================
 
   if(!is.null(tester)){
-    which.miss <- res[rnames==tester,]!=label.heter&!is.na(res[rnames==tester,])| knames
-    res <- res[,which.miss]
+    which.miss <- gpData$geno[rnames==tester,]!=label.heter&!is.na(gpData$geno[rnames==tester,])| knames
+    gpData$geno <- gpData$geno[,which.miss]
     cnames <- cnames[which.miss]; knames <- knames[which.miss]
     if(sum(!which.miss) > 0){
       if (verbose) cat("   step 3 :",sum(!which.miss),"marker(s) discarded because heterozygousity at tester locus or \n          missing values of the tester\n")
@@ -283,12 +284,12 @@ codeGeno <- function(gpData,impute=FALSE,impute.type=c("random","family","beagle
   if(!is.null(maf)){
     if(maf<0 | maf>1) stop("'maf' must be in [0,1]")
     if(is.null(tester)){
-      which.maf <- colMeans(res,na.rm=TRUE)>=2*maf | knames
+      which.maf <- colMeans(gpData$geno,na.rm=TRUE)>=2*maf | knames
     } else {
-      which.maf <- colMeans(res,na.rm=TRUE)>=maf & colMeans(res,na.rm=TRUE)<=1-maf  | knames
+      which.maf <- colMeans(gpData$geno,na.rm=TRUE)>=maf & colMeans(gpData$geno,na.rm=TRUE)<=1-maf  | knames
     }
     if (verbose) cat("   step 4  :",sum(!which.maf),"marker(s) removed with maf <",maf,"\n")
-    res <- res[,which.maf]
+    gpData$geno <- gpData$geno[,which.maf]
     cnames <- cnames[which.maf]; knames <- knames[which.maf]
     # update map
     if(!is.null(gpData$map)) gpData$map <- gpData$map[which.maf,]
@@ -303,8 +304,8 @@ codeGeno <- function(gpData,impute=FALSE,impute.type=c("random","family","beagle
   #============================================================
 
   if(!is.null(tester)){
-    which.miss <- res[rnames==tester,] != 2 | knames
-    res <- res[,which.miss]
+    which.miss <- gpData$geno[rnames==tester,] != 2 | knames
+    gpData$geno <- gpData$geno[,which.miss]
     cnames <- cnames[which.miss]; knames <- knames[which.miss]
     if(sum(!which.miss) > 0){
       if (verbose) cat("   step 5  :",sum(!which.miss),"marker(s) discarded for which the tester has the minor allele\n")
@@ -320,11 +321,11 @@ codeGeno <- function(gpData,impute=FALSE,impute.type=c("random","family","beagle
   #============================================================
 
   if(!is.null(tester)){
-    res[res == 2] <- NA
-    res <- matrix(as.numeric(res), nrow = n)
+    gpData$geno[gpData$geno == 2] <- NA
+    gpData$geno <- matrix(as.numeric(gpData$geno), nrow = n)
     if(!is.null(nmiss)){
-      which.miss <- apply(is.na(res),2,mean,na.rm=TRUE) <= nmiss |knames
-      res <- res[,which.miss]
+      which.miss <- apply(is.na(gpData$geno),2,mean,na.rm=TRUE) <= nmiss |knames
+      gpData$geno <- gpData$geno[,which.miss]
       cnames <- cnames[which.miss]; knames <- knames[which.miss]
       if (verbose) cat("   step 6  :",sum(!which.miss),"marker(s) discarded with >",nmiss*100,"% false genotyping values \n")
       # update map
@@ -339,27 +340,27 @@ codeGeno <- function(gpData,impute=FALSE,impute.type=c("random","family","beagle
   #============================================================
 
   # initialize counter
-    cnt1 <- rep(0,ncol(res))   # for nr. of imputations with family structure
-    cnt2 <- rep(0,ncol(res))    # for nr. of beagle imputations
-    cnt3 <- rep(0,ncol(res))    # for nr. of random imputations
+    cnt1 <- rep(0,ncol(gpData$geno))   # for nr. of imputations with family structure
+    cnt2 <- rep(0,ncol(gpData$geno))    # for nr. of beagle imputations
+    cnt3 <- rep(0,ncol(gpData$geno))    # for nr. of random imputations
     names(cnt1) <- names(cnt2) <- names(cnt3) <- cnames
 
   # start of imputing
   if(impute){
     set.seed(SEED[1])
     # number of markers
-    M <- ncol(res)
+    M <- ncol(gpData$geno)
     if(M==0) stop(" no markers remained after step 1 (to many missing values)")
     if (verbose) cat("   step 7  : Imputing of missing values \n")
 
     # number of missing values
-    nmv <- sum(is.na(res))
+    nmv <- sum(is.na(gpData$geno))
 
     ###########################################################################
     # if impute.type="fix", replace missing values according to specified value
     ###########################################################################
     if(impute.type=="fix"){
-      res[is.na(res)] <- replace.value
+      gpData$geno[is.na(gpData$geno)] <- replace.value
       if (verbose) cat("   step 7a : Replace missing values by",replace.value," \n")
     }
     #########################################################
@@ -370,13 +371,14 @@ codeGeno <- function(gpData,impute=FALSE,impute.type=c("random","family","beagle
       # initialize counter (- number of heterozygous values)
       # loop over all markers
       probList <- list(c(1), c(.5,.5), c(.25,.5,.25))
-      vec.cols <- (1:M)[is.na(colSums(res, na.rm = FALSE))]
+      vec.cols <- (1:M)[is.na(colSums(gpData$geno, na.rm = FALSE))]
+      nFam <- table(popStruc)
+      vec.big <- popStruc %in% names(nFam)[nFam > minFam]
       for (j in vec.cols){
-        if(j==vec.cols[1]) ptm <- proc.time()[3]
-        if(sum(!is.na(res[,j]))>0){
-          try({# compute population structure  as counts
-               poptab <- table(popStruc,res[,j])
-               nFam <- table(popStruc)
+        if(j==vec.cols[1]) ptm <- Sys.time()
+        if(sum(!is.na(gpData$geno[,j]))>0){
+#          try({# compute population structure  as counts
+               poptab <- table(popStruc[vec.big],gpData$geno[vec.big,j])
                rS <- rowSums(poptab)
                # compute otherstatistics
                major.allele <- unlist(attr(poptab,"dimnames")[[2]][apply(poptab,1,which.max)])
@@ -385,39 +387,46 @@ codeGeno <- function(gpData,impute=FALSE,impute.type=c("random","family","beagle
                polymorph2 <- rS > minFam
                polymorph[!polymorph2] <- TRUE
                # count missing values
-               nmissfam <- tapply(is.na(res[,j]),popStruc,sum)
+               nmissfam <- tapply(is.na(gpData$geno[vec.big,j]),popStruc[vec.big],sum)
                # must be a named list
                names(major.allele) <- names(polymorph)
                # loop over all families
                for (i in rownames(poptab)[nmissfam > 0]){
                  # impute values for impute.type="family" : all missing genotypes
-                 allTab <- table(res[popStruc == i, j])
+                 allTab <- table(gpData$geno[popStruc[vec.big] %in% i, j])
                  if(length(allTab) == 0 & noHet) {
                    allTab <- table(c(0,2))
                  } else if(all(names(allTab) == c(0, 2)) & !noHet)  allTab <- table(c(0,1,1,2))
                  if (impute.type=="family"){
-                   res[is.na(res[,j]) & popStruc == i ,j] <- ifelse(length(allTab)>1, sample(as.numeric(names(allTab)),size=nmissfam[i],prob=probList[[length(allTab)]],replace=TRUE),as.numeric(names(allTab)))
+                   gpData$geno[is.na(gpData$geno[,j]) & popStruc %in% i ,j] <- ifelse(length(allTab)>1,
+                                                                    sample(as.numeric(names(allTab)),size=nmissfam[as.character(i)],prob=probList[[length(allTab)]],replace=TRUE),
+                                                                    as.numeric(names(allTab)))
                    # update counter
-                   if(polymorph[i]) cnt3[j] <- cnt3[j] + nmissfam[i] else cnt1[j] <- cnt1[j] + nmissfam[i]
+                   if(polymorph[as.character(i)]) cnt3[j] <- cnt3[j] + nmissfam[as.character(i)] else cnt1[j] <- cnt1[j] + nmissfam[as.character(i)]
                  }
-                 if(impute.type %in%c("beagleAfterFamily", "beagleAfterFamilyNoRand")){
+                 if(impute.type %in%c("beagleAfterFamily")){
                    if (is.na(gpData$map$pos[j])){     # if no position is available use family algorithm
-                     res[is.na(res[,j]) & popStruc == i ,j] <- ifelse(length(allTab)>1, sample(as.numeric(names(allTab)),size=nmissfam[i],prob=probList[[length(allTab)]],replace=TRUE),as.numeric(names(allTab)))
+                     gpData$geno[is.na(gpData$geno[,j]) & popStruc %in% i ,j] <- ifelse(length(allTab)>1,
+                                                                      sample(as.numeric(names(allTab)),size=nmissfam[as.character(i)],prob=probList[[length(allTab)]],replace=TRUE),
+                                                                      as.numeric(names(allTab)))
                      # update counter
-                     if(polymorph[i]) cnt3[j] <- cnt3[j] +  nmissfam[i] else cnt1[j] <- cnt1[j] +  nmissfam[i]
+                     if(polymorph[as.character(i)]) cnt3[j] <- cnt3[j] +  nmissfam[as.character(i)] else cnt1[j] <- cnt1[j] +  nmissfam[as.character(i)]
                    } else { # use Beagle and impute NA for polymorphic families
-                     if(!polymorph[i]){
+                     if(!polymorph[as.character(i)]){
                        # impute values for impute.type="beagleAfterfamily"  : only monomorph markers
-                       res[is.na(res[,j]) & popStruc == i ,j] <- as.numeric(rep(major.allele[i],nmissfam[i]))
                        # update counter
-                       cnt1[j] <- cnt1[j] + nmissfam[i]
+                       cnt1[j] <- cnt1[j] + nmissfam[as.character(i)]
                      }
                    }
                  }
                }
-               if(j==ceiling(length(vec.cols)/100)) if(verbose) cat("         approximative run time for imputation by family information ",(proc.time()[3] - ptm)*99," seconds ... \n",sep="")
-          }) # end try
-        }   # end of if(sum(!is.na(res[,j]))>0)
+               if(j==ceiling(length(vec.cols)/50))
+                 if(verbose)  cat("         approximative run time for imputation by family information ",
+                                  paste(round(as.numeric(difftime(Sys.time(), ptm)*50)), digits=1, " ",
+                                  units(difftime(Sys.time(), ptm))," ... \n",sep=""))
+
+#          }, silent= !verbose) # end try
+        }   # end of if(sum(!is.na(gpData$geno[,j]))>0)
       } # end of marker loop
     }
 
@@ -431,44 +440,44 @@ codeGeno <- function(gpData,impute=FALSE,impute.type=c("random","family","beagle
         chr <-  unique(gpData$map$chr)
         chr <- chr[!is.na(chr)]
         if(!is.null(tester))
-          res <- res*2
-        rownames(res) <- rownames(gpData$geno)
-        colnames(res) <- cnames
-        cnt2 <- apply(is.na(res),2,sum)
+          gpData$geno <- gpData$geno*2
+        rownames(gpData$geno) <- rnames
+        colnames(gpData$geno) <- cnames
+        cnt2 <- apply(is.na(gpData$geno),2,sum)
         # loop over chromosomses
+        beagleDir <- paste("beagle", as.numeric(as.Date(Sys.time())), round(as.numeric(Sys.time())%%(24*3600)), sep="")
+        if(!beagleDir %in% list.files()){
+          dir.create(beagleDir)
+        } else if(length(list.files(beagleDir)) > 0 ) {
+           file.remove(paste(beagleDir, list.files(beagleDir), sep="/"))
+        }
         for (lg in seq(along=chr)){
-          if(lg==1) ptm <- proc.time()[3]
+          if(lg==1) ptm <- Sys.time()
           if(verbose) cat("          chromosome ", as.character(chr)[lg], "\n")
           sel <- unique(c(rownames(gpData$map[is.na(gpData$map$pos) | gpData$map$chr != chr[lg] | !rownames(gpData$map) %in% cnames ,]),
                           colnames(gpData$geno)[!colnames(gpData$geno) %in% cnames]))
           markerTEMPbeagle <- discard.markers(gpData,which=sel)
-          markerTEMPbeagle$geno <- res[, colnames(markerTEMPbeagle$geno)]
 
           # write input files for beagle
           pre <- paste("chr",chr[lg],sep="")
           pre <- gsub(" ", "_", pre, fixed=TRUE)
           # create new directory "beagle" for beagle input and output files
-          if(!"beagle" %in% list.files()){
-            dir.create("beagle")
-          } else if(lg == 1 & length(list.files("beagle")) > 0 ) {
-             file.remove(paste("beagle", list.files("beagle"), sep="/"))
-          }
-          write.vcf(markerTEMPbeagle,paste(file.path(getwd(),"beagle/"),"/",prefix=pre, "input.vcf", sep=""))
+          write.vcf(markerTEMPbeagle,paste(file.path(getwd(), beagleDir),"/",prefix=pre, "input.vcf", sep=""))
           if(noHet){
-          output <- system(paste("java -Xmx1000m -jar ",
+          output <- system(paste("java -Xmx3000m -jar ",
                            shQuote(paste(sort(path.package()[grep("synbreed", path.package())])[1], "/java/beagle.r1399.jar", sep="")),
                            # caution with more than one pacakge with names synbreed*, assume synbreed to be the first one
-                           " gt=beagle/", pre, "input.vcf usephase=true out=beagle/", pre, "out", sep=""),
+                           " gt=", beagleDir, "/", pre, "input.vcf usephase=true out=", beagleDir, "/", pre, "out", sep=""),
                            intern=!showBeagleOutput)
           } else {
-          output <- system(paste("java -Xmx1000m -jar ",
+          output <- system(paste("java -Xmx3000m -jar ",
                            shQuote(paste(sort(path.package()[grep("synbreed", path.package())])[1], "/java/beagle.r1399.jar", sep="")),
                            # caution with more than one pacakge with names synbreed*, assume synbreed to be the first one
-                           " gt=beagle/", pre, "input.vcf out=beagle/", pre, "out", sep=""),
+                           " gt=", beagleDir, "/", pre, "input.vcf out=", beagleDir, "/", pre, "out", sep=""),
                            intern=!showBeagleOutput)
           }
           # read data from beagle
-          resTEMP <- read.vcf2matrix(file=gzfile(paste("beagle/",pre,"out.vcf.gz",sep="")), FORMAT="DS", IDinRow=TRUE)
+          resTEMP <- read.vcf2matrix(file=gzfile(paste(beagleDir, "/",pre,"out.vcf.gz",sep="")), FORMAT="DS", IDinRow=TRUE)
           mode(resTEMP) <- "numeric"
 
 
@@ -481,12 +490,15 @@ codeGeno <- function(gpData,impute=FALSE,impute.type=c("random","family","beagle
           }
 
           if (length(sel)>0) {
-            res[,!colnames(res) %in% sel] <- resTEMP
+            gpData$geno[,!colnames(gpData$geno) %in% sel] <- resTEMP
           } else {
-            res <- resTEMP
+            gpData$geno <- resTEMP
           }
-          if(lg==1) if(verbose) cat("\n         approximative run time for beagle imputation ",
-                                        (proc.time()[3] - ptm)/ncol(markerTEMPbeagle$geno)*ncol(res)," seconds ... \n\n",sep="")
+          if(lg==1)
+            if(verbose)  cat("         approximative run time for imputation by beagle ",
+                             paste(round(as.numeric(difftime(Sys.time(), ptm)/ncol(markerTEMPbeagle$geno)*ncol(gpData$geno)), digits=1), " ",
+                             units(difftime(Sys.time(), ptm))," ... \n",sep=""))
+
         }
       }
 
@@ -496,22 +508,22 @@ codeGeno <- function(gpData,impute=FALSE,impute.type=c("random","family","beagle
       if(impute.type %in% c("random", "beagle", "beagleAfterFamily")){
         if (verbose) cat("   step 7d : Random imputing of missing values \n")
         # initialize counter (- number of heterozygous values)
-        for (j in (1:M)[apply(is.na(res), 2, sum)>0]){
-          cnt3[j] <-  cnt3[j] + sum(is.na(res[,j]))
+        for (j in (1:M)[apply(is.na(gpData$geno), 2, sum)>0]){
+          cnt3[j] <-  cnt3[j] + sum(is.na(gpData$geno[,j]))
           # estimation of running time after the first iteration
           if(j==1) ptm <- proc.time()[3]
-            p <- mean(res[,j],na.rm=TRUE)/2  # minor allele frequency
+            p <- mean(gpData$geno[,j],na.rm=TRUE)/2  # minor allele frequency
             if(noHet){        # assuming only 2 homozygous genotypes
-              res[is.na(res[,j]),j] <- sample(c(0,2),size=sum(is.na(res[,j])),prob=c(1-p,p),replace=TRUE)
+              gpData$geno[is.na(gpData$geno[,j]),j] <- sample(c(0,2),size=sum(is.na(gpData$geno[,j])),prob=c(1-p,p),replace=TRUE)
             } else {                            # assuming 3 genotypes
-              res[is.na(res[,j]),j] <- sample(c(0,1,2),size=sum(is.na(res[,j])),prob=c((1-p)^2,2*p*(1-p),p^2),replace=TRUE)
+              gpData$geno[is.na(gpData$geno[,j]),j] <- sample(c(0,1,2),size=sum(is.na(gpData$geno[,j])),prob=c((1-p)^2,2*p*(1-p),p^2),replace=TRUE)
             }
             if(j==ceiling(M/100)) if(verbose) cat("         approximate run time for random imputation ",(proc.time()[3] - ptm)*99," seconds \n",sep=" ")
           }
           # update counter for Beagle, remove those counts which where imputed ranomly
           if(impute.type == "beagle") cnt2 <- cnt2-cnt3
         }
-        if(!is.null(tester) & impute.type %in% c("random","beagle", "beagleAfterFamily")) res <- res/2
+        if(!is.null(tester) & impute.type %in% c("random","beagle", "beagleAfterFamily")) gpData$geno <- gpData$geno/2
 
 
     #============================================================
@@ -519,16 +531,16 @@ codeGeno <- function(gpData,impute=FALSE,impute.type=c("random","family","beagle
     #============================================================
 
     # recode again if allele frequeny changed to to imputing
-    if(any(colMeans(res,na.rm=TRUE)>1)){
+    if(any(colMeans(gpData$geno,na.rm=TRUE)>1)&reference.allele[1]=="minor"){
       if (verbose) cat("   step 8  : Recode alleles due to imputation \n")
-      res[,which(colMeans(res,na.rm=TRUE)>1)] <- 2 - res[,which(colMeans(res,na.rm=TRUE)>1)]
+      gpData$geno[,which(colMeans(gpData$geno,na.rm=TRUE)>1)] <- 2 - gpData$geno[,which(colMeans(gpData$geno,na.rm=TRUE)>1)]
     } else{
       if (verbose) cat("   step 8  : No recoding of alleles necessary after imputation \n")
     }
   }
   # update report list
   if(print.report){
-    major[which(colMeans(res,na.rm=TRUE)>1)] <- minor[which(colMeans(res,na.rm=TRUE)>1)]
+    major[which(colMeans(gpData$geno,na.rm=TRUE)>1)] <- minor[which(colMeans(gpData$geno,na.rm=TRUE)>1)]
   }
 
   #============================================================
@@ -537,10 +549,10 @@ codeGeno <- function(gpData,impute=FALSE,impute.type=c("random","family","beagle
 
   if(!is.null(maf) & impute){
     if(maf<0 | maf>1) stop("'maf' must be in [0,1]")
-    if(is.null(tester)) which.maf <- colMeans(res,na.rm=TRUE)>=2*maf | knames else
-      which.maf <- colMeans(res,na.rm=TRUE)>=maf & colMeans(res,na.rm=TRUE)<=1-maf | knames
+    if(is.null(tester)) which.maf <- colMeans(gpData$geno,na.rm=TRUE)>=2*maf | knames else
+      which.maf <- colMeans(gpData$geno,na.rm=TRUE)>=maf & colMeans(gpData$geno,na.rm=TRUE)<=1-maf | knames
     if (verbose) cat("   step 9  :",sum(!which.maf),"marker(s) removed with maf <",maf,"\n")
-    res <- res[,which.maf]
+    gpData$geno <- gpData$geno[,which.maf]
     cnames <- cnames[which.maf]; knames <- knames[which.maf]
     # update map
     if(!is.null(gpData$map)) gpData$map <- gpData$map[which.maf,]
@@ -557,15 +569,15 @@ codeGeno <- function(gpData,impute=FALSE,impute.type=c("random","family","beagle
 
   if(!keep.identical){
     set.seed(SEED[2])
-    cnms <- sample(1:ncol(res))
-    which.duplicated <- duplicated(res[, cnms],MARGIN=2)
+    cnms <- sample(1:ncol(gpData$geno))
+    which.duplicated <- duplicated(gpData$geno[, cnms],MARGIN=2)
     which.duplicated <- cnames %in% (cnames[cnms])[which.duplicated]
-    rev.which.duplicated <- duplicated(res[, cnms],MARGIN=2, fromLast=TRUE)
+    rev.which.duplicated <- duplicated(gpData$geno[, cnms],MARGIN=2, fromLast=TRUE)
     rev.which.duplicated <- cnames %in% (cnames[cnms])[rev.which.duplicated]
     rev.which.duplicated[which.duplicated] <- FALSE
     if(impute){
        if(sum(which.duplicated) >0){
-        mat.ld <- cor(res[, which.duplicated], res[, rev.which.duplicated], use="pairwise.complete.obs")
+        mat.ld <- cor(gpData$geno[, which.duplicated], gpData$geno[, rev.which.duplicated], use="pairwise.complete.obs")
         df.ld <- data.frame(kept=rep(cnames[which.duplicated], ncol(mat.ld)),
                             removed=rep(cnames[rev.which.duplicated], each=nrow(mat.ld)),
                             ld=as.numeric(mat.ld),
@@ -575,30 +587,30 @@ codeGeno <- function(gpData,impute=FALSE,impute.type=c("random","family","beagle
         rm(mat.ld)
       } else df.ld <- data.frame(kept=as.character(), removed=as.character())
     } else {# end of imputed step
-      if(!all(!is.na(res))){
+      if(!all(!is.na(gpData$geno))){
         if(sum(which.duplicated) >0){
-          res[is.na(res)] <- 3
-          mat.ld <- cor(res[, which.duplicated], res[, rev.which.duplicated])
+          gpData$geno[is.na(gpData$geno)] <- 3
+          mat.ld <- cor(gpData$geno[, which.duplicated], gpData$geno[, rev.which.duplicated])
           df.ld <- data.frame(kept=rep(cnames[which.duplicated], each=nrow(mat.ld)),
                               removed=rep(cnames[rev.which.duplicated], ncol(mat.ld)),
                               ld=as.numeric(mat.ld),
                               stringsAsFactors=FALSE)
           df.ld <- df.ld[df.ld$ld>1-1e-14,]
           df.ld$ld <- NULL
-          res[res==3] <- NA
+          gpData$geno[gpData$geno==3] <- NA
           rm(mat.ld)
         } else df.ld <- data.frame(kept=as.character(), removed=as.character())
 #        cat(str(df.ld, "\n"))
-	    which.miss <- apply(is.na(res),2,sum)>0 	
+	    which.miss <- apply(is.na(gpData$geno),2,sum)>0 	
 	    which.miss <- (1:length(which.miss))[which.miss] 	
-  	  if(length(which.miss[which.miss]) == ncol(res))
+  	  if(length(which.miss[which.miss]) == ncol(gpData$geno))
           which.miss <- which.miss[1:(length(which.miss)-1)] 	
         if(is.null(keep.list)){
           for(i in which.miss){ 	
             if(which.duplicated[i]) next 	
-            for(j in ((i+1):ncol(res))[!which.duplicated[(i+1):ncol(res)]]){ 	
-              if(all(res[, i] == res[, j], na.rm = TRUE)){ 	
-                if(sum(is.na(res[, i])) >= sum(is.na(res[, j]))){ 	
+            for(j in ((i+1):ncol(gpData$geno))[!which.duplicated[(i+1):ncol(gpData$geno)]]){ 	
+              if(all(gpData$geno[, i] == gpData$geno[, j], na.rm = TRUE)){ 	
+                if(sum(is.na(gpData$geno[, i])) >= sum(is.na(gpData$geno[, j]))){ 	
                   which.duplicated[i] <- TRUE 	
                   df.ld <- rbind(df.ld, data.frame(kept=cnames[j], removed=cnames[i]))
                   break 	
@@ -612,14 +624,14 @@ codeGeno <- function(gpData,impute=FALSE,impute.type=c("random","family","beagle
         } else {
           for(i in which.miss){ 	
             if(which.duplicated[i]) next 	
-            for(j in ((i+1):ncol(res))[!which.duplicated[(i+1):ncol(res)]]){ 	
-              if(all(res[, i] == res[, j], na.rm = TRUE)){
+            for(j in ((i+1):ncol(gpData$geno))[!which.duplicated[(i+1):ncol(gpData$geno)]]){ 	
+              if(all(gpData$geno[, i] == gpData$geno[, j], na.rm = TRUE)){
                 if(knames[i]){# knames is logical vector for keep.list. Faster than testing if cnames[i] in keep.list!
                   if(knames[j]) next else which.duplicated[j] <- TRUE
                 } else {
                   if(knames[j]){ which.duplicated[i] <- TRUE
                   } else {
-                    if(sum(is.na(res[, i])) >= sum(is.na(res[, j]))){ 	
+                    if(sum(is.na(gpData$geno[, i])) >= sum(is.na(gpData$geno[, j]))){ 	
                       which.duplicated[i] <- TRUE 	
                       df.ld <- rbind(df.ld, data.frame(kept=cnames[j], removed=cnames[i]))
                       break 	
@@ -636,7 +648,7 @@ codeGeno <- function(gpData,impute=FALSE,impute.type=c("random","family","beagle
         if(is.na(df.ld[1,1])) df.ld <- df.ld[-1,]
       } else {# end of missing value step
         if(sum(which.duplicated) >0){
-          mat.ld <- cor(res[, which.duplicated], res[, rev.which.duplicated])
+          mat.ld <- cor(gpData$geno[, which.duplicated], gpData$geno[, rev.which.duplicated])
           df.ld <- data.frame(kept=rep(colnames(mat.ld), nrow(mat.ld)),
                               removed=rep(rownames(mat.ld), each=ncol(mat.ld)),
                               ld=as.numeric(mat.ld),
@@ -647,8 +659,8 @@ codeGeno <- function(gpData,impute=FALSE,impute.type=c("random","family","beagle
         } else df.ld <- data.frame(kept=as.character(), removed=as.character())
       }
     } # end of not imputed step
-    res <- res[, !which.duplicated]
-    attr(res, "identical") <- df.ld
+    gpData$geno <- gpData$geno[, !which.duplicated]
+    attr(gpData$geno, "identical") <- df.ld
     cnames <- cnames[!which.duplicated]
     if (verbose) cat("   step 10 :",sum(which.duplicated),"duplicated marker(s) removed \n")
     # update map
@@ -663,8 +675,8 @@ codeGeno <- function(gpData,impute=FALSE,impute.type=c("random","family","beagle
   #============================================================
 
   if(!is.null(tester)){
-    which.fixed <- apply(res, 2, sum) == nrow(res)-1 | knames
-    res <- res[,!which.fixed]
+    which.fixed <- apply(gpData$geno, 2, sum) == nrow(gpData$geno)-1 | knames
+    gpData$geno <- gpData$geno[,!which.fixed]
     cnames <- cnames[!which.fixed]; knames <- knames[!which.fixed]
     if(!is.null(gpData$map)) gpData$map <- gpData$map[!which.fixed,]
     if (verbose)
@@ -679,16 +691,16 @@ codeGeno <- function(gpData,impute=FALSE,impute.type=c("random","family","beagle
   # step 11 - restoring original data format
   #============================================================
 
-  rownames(res) <- rnames
-  colnames(res) <- cnames
+  rownames(gpData$geno) <- rnames
+  colnames(gpData$geno) <- cnames
   if(orgFormat == "matrix"){
-    res <- matrix(res,nrow=n)
+    gpData$geno <- matrix(gpData$geno,nrow=n)
   }
   if(orgFormat == "data.frame"){
-    res  <- as.data.frame(res)
+    gpData$geno  <- as.data.frame(gpData$geno)
   }
 
-  if (verbose) cat("   End     :",ncol(res),"marker(s) remain after the check\n")
+  if (verbose) cat("   End     :",ncol(gpData$geno),"marker(s) remain after the check\n")
 
   #============================================================
   # print summary of imputation
@@ -705,9 +717,8 @@ codeGeno <- function(gpData,impute=FALSE,impute.type=c("random","family","beagle
 
   # overwrite original genotypic data
   if(orgFormat == "gpData") {
-    gpData$geno <- res
     gpData$info$codeGeno <- TRUE
-  } else gpData <- res
+  } else gpData <- gpData$geno
 
   if(print.report){
     if (verbose) cat("  Writing report to file 'SNPreport.txt' \n")
